// Step 0: unzip RAR files, convert to txt, and then to dta files

* 0) Preliminaries
*** Cleaning
	clear
	cls
	set more off
	set rmsg on																	// Keeps track of length of process
	
*** Install codes
   ssc install fs
   ssc install confirmdir
   ssc install chunky
   
*** Directories
global unzip "C:\Program Files\7-Zip\7z"				      					// Location of 7z executable program
global loc "CHANGE TO RELEVANT DRIVE"											// CHANGE DRIVE WHERE FORLDERS ARE LOCATED
global raw "$loc\financials\zip" 												// Location of zip files (SUGGESTION: COPY INTO THIS FOLDER ONLY THE ZIP FILES THAT YOU WANT TO CONVERT)
global txt "$loc\financial\txt"													// Output text files
global dta "$loc\financials\dta"												// Output data files 

global codes "$loc\processing_codes"											// Locataion to save log files
	
// SECTION 1: THE INDUSTRY-GLOBAL FINANCIALS AND RATIOS - USD FILE IS TOO BIG TO IMPORT THE FULL TEXT FILE SO IT NEEDS TO BE BROKEN INTO PIECES
* 1) Unzip files
disp "Generate list of individual files to process"
clear
set obs 1 
	gen fname = "IndustryGlobalFinancialsAndRatiosUSD"
	gen id = 1

disp "List of files in processing loop"
	list fname
save "$loc\financials\files.dta", replace
	qui sum id
	
disp "Beginning to loop 1 through files: saving the data"		
forval i = `r(min)'(1)`r(max)'{
	use "$loc\financials\files.dta", clear
	keep if id == `i'
	global file `=fname[1]'
	 disp "Processing $file"
	// ! tells Stata that command is (in this case) meant to be run through Windows command line
	 disp "Unzip file"
	! "$unzip" e -y "$raw\\$file.rar" -o"$txt\\" -r
} 

* 2) Save text files into DTA format
use "$loc\financials\files.dta", clear			
	qui sum id
forval i = `r(min)'(1)`r(max)'{
	use "$loc\financials\files.dta", clear
	keep if id == `i'
	global file `=fname[1]'
	 disp "Processing $file"

	 disp "Import text file"
	 chunky using  "$txt\\$file.txt", replace chunksize(2GB) header(include) stub("$txt\chunky\\$file")
		clear
	fs "$txt\chunky\*.txt"
	foreach f in `r(files)' {
		import delimited "$txt\chunky\\`f'", varnames(1) clear
		disp "Save data file: `f'"
		save "$dta\intermediate\\`f'.dta", replace	
		erase "$txt\chunky\\`f'"
	}
}

* 3) Clean up
erase "$loc\financials\files.dta"

// SECTION 2: OTHER FINANCIAL FILES ARE SMALL ENOUGH THAT THEY DO NOT NEED TO BE BROKEN DOWN INTO CHUNKS. 
* 1) Unzip files
disp "Generate list of individual files to process"
clear
set obs 6
gen id = _n
gen fname = ""
	replace fname = "CashFlowUSIndustriesUSD" if id == 1
	replace fname = "CashFlowNonUSIndustriesUSD" if id == 2
	replace fname = "BanksGlobalFinancialsRatiosUSD" if id == 3
	replace fname = "DetailedFormatIndustriesUSD" if id == 4
	replace fname = "InsurancesGlobalFinancialsAndRatiosUSD" if id == 5
	replace fname = "KeyFinancialsUSD" if id == 6

disp "List of files in processing loop"
	list fname
save "$loc\financials\files.dta", replace
	qui sum id

disp "Beginning to loop 1 through files: saving the data"		
forval i = `r(min)'(1)`r(max)'{
	use "$loc\financials\files.dta", clear
	keep if id == `i'
	global file `=fname[1]'
	 disp "Processing $file"
	// ! tells Stata that command is (in this case) meant to be run through Windows command line
	 disp "Unzip file"
	! "$unzip" e -y "$raw\\$file.rar" -o"$txt\\" -r
} 	

* 2) Save text files into DTA format
use "$loc\financials\files.dta", clear			
	qui sum id
forval i = `r(min)'(1)`r(max)'{
	use "$loc\financials\files.dta", clear
	keep if id == `i'
	global file `=fname[1]'
	disp "Processing $file"
	
	disp "Import text file"
	import delimited "$txt\\$file.txt", varnames(1) clear
	disp "Save data file: $file"
	save "$dta\intermediate\\$file.dta", replace	
}

* 3) Clean up
erase "$loc\financials\files.dta"

