// Step 3: Combine firm description files and save them in the appropriate country folder

* 0) Preliminaries
*** Cleaning
	clear
	cls
	set more off
	set rmsg on																	// Keeps track of length of process
	
*** Install codes
   ssc install fs
   ssc install confirmdir
   ssc install chunky
   
*** Directories
global unzip "C:\Program Files\7-Zip\7z"				      					// Location of 7z executable program
global loc "CHANGE TO RELEVANT DRIVE"											// CHANGE DRIVE WHERE FORLDERS ARE LOCATED
global raw "$loc\ownership\zip" 												// Location of zip files (SUGGESTION: COPY INTO THIS FOLDER ONLY THE ZIP FILES THAT YOU WANT TO CONVERT. FOR OWNERSHIP PROCESS LINKS FILES & ENTITIES FILE)
global txt "$loc\ownership\txt"													// Output text files
global dta "$loc\ownership\dta"													// Output data files 

global codes "$loc\processing_codes"											// Locataion to save log files
	
*** Create Country Folders (for stacked final files)
use "$loc\countries_ISO.dta", clear
	levelsof country, local(cntry)
foreach c of local cntry {
	confirmdir "$dta\appended_countries\\`c'"
	if r(confirmdir) == "170" {
		mkdir "$dta\appended_countries\\`c'"
	}
}

* 1) Create blank files
use "$loc\countries_ISO.dta", clear
	levelsof country, local(cntry)
clear

fs "$dta\intermdiate\*.dta"
	return list
set obs 1 
	gen fn = ""
	gen id = 0
foreach f in `r(files)' {
	insobs 1
	replace id = id[_n-1] + 1 if _n > 1
	local name `f'
	replace fn = "`name'" if _n == _N
}
drop if id == 0

split fn, p(".") gen(fname)
	keep fname1 id
	rename fname1 fname
	keep if strpos(fname,"links") > 0
	drop id
	gen id = _n

*** Each links files will have multiple sub-files and this set of commands will help differentiate among all the files 
gen len = strlen(fname)
	replace len = len-4

gen core = substr(fname,1,len)
	egen id_core = group(core)

save "$loc\ownership\link_files.dta", replace

bys id_core: keep if _n == 1
save "$loc\ownership\link_core.dta", replace

*** Generate the blank files
use "$loc\ownership\link_core.dta", clear
	sum id_core
	return list
forval i = `r(min)'(1)`r(max)' {
	use "$loc\ownership\link_core.dta", clear
	keep if id_core == `i'
	global core `=core[1]'
	disp "$core"
	foreach c of local cntry {
		// Save by country of subsidiary (so contains information on all its shareholders, regardless of shareholder country)
		clear
		set obs 1
		gen bvdid = ""
		save "$dta\appended_countries\\`c'\SHARE_`c'_$core.dta", replace
		
		// Save by country of shareholder (so contains all its subsidiaries, regardless of subsidiary country)
		clear
		set obs 1
		gen bvdid = ""
		save "$dta\appended_countries\\`c'\SUB_`c'_$core.dta", replace
	}
}

* 2) Save the ownership data (annual files)
use "$loc\ownership\link_files.dta", clear
	qui sum id
	return list
forval i = `r(min)'(1)`r(max)'{
	use "$loc\ownership\link_files.dta", clear
	keep if id == `i'
	global file `=fname[1]'
	disp "$file"
	global core `=core[1]'
	disp "$core"
	use "$dta\intermediate\\$file.txt.dta", clear

	rename subsidiarybvdid bvdid
	gen file = $core
		replace file = 2016 if file == "current"								// last year will be coded as current. Make sure to change this to your current year
		destring file, replace
			
	gen ISO_final_subsidiary = substr(bvdid,1,2)
	gen ISO_final_shareholder = substr(shareholderbvdid, 1,2)
		
	// Save by country of subsidiary
	levelsof ISO_final_subsidiary, local(cntry)
	foreach c of local cntry {
	preserve
		keep if ISO_final_subsidiary == "`c'"
		append using "$dta\appended_countries\\`c'\SHARE_`c'_$core.dta"
			drop if bvdid == ""
		save "$dta\appended_countries\\`c'\SHARE_`c'_$core.dta", replace
	restore
	}	
	
	// Save by country of shareholder
	levelsof ISO_final_shareholder, local(cntry)
	foreach c of local cntry {
	preserve
		keep if ISO_final_shareholder == "`c'"
		append using "$dta\appended_countries\\`c'\SUB_`c'_$core.dta"
			drop if bvdid == ""
		save "$dta\appended_countries\\`c'\SUB_`c'_$core.dta", replace
	restore
	}	
}

* 3) Erase
erase "$loc\ownership\link_files.dta"

* 4) Combine annual files to form a panel (both for subsidiary and shareholder files) 	
use "$loc\countries_ISO.dta", clear
	levelsof country, local(cntry)
	
*** NOTE: RENAME THE CURRENT FILE TO AN ACTUAL YEAR FILE SO THAT IT CAN BE USED IN THE LOOP BELOW
foreach c of local cntry {
	foreach s in SHARE SUB {
		use "$dta\appended_countries\\`c'\\`s'_`c'_links_current.dta", clear
		save "$dta\appended_countries\\`c'\\`s'_`c'_links_2016.dta", replace
		erase "$dta\appended_countries\\`c'\\`s'_`c'_links_current.dta"
	}
}	

*** NOTE: NOT ALL COUNTRIES HAVE THE SAME COVERAGE (SOME DON'T HAVE DATA STARTING IN 2007). THE FOLLOWING PROCEDURE TAKES THIS INTO ACCOUNT		
foreach c of local cntry {
	foreach s in SHARE SUB {
		disp "`c' `s'"
		
		*** Figure out first year for which the country has data
		local loop = 1
		local initial 0
		forval i = 2007(1)2016 {
			if `loop' == 1 {
				disp "CHECKING YEAR = `i'"
				use "$dta\appended_countries\\`c'\\`s'_`c'_links_`i'.dta", clear
				qui count
				if `r(N)' > 1 {
						local initial `i'
						local loop = 0
				}
			}
		}
		if `initial' == 0 {
			disp "NO OBSERVATIONS FOR FILE `s' FOR COUNTRY `c'" 
		}
		if `initial' != 0 {		
			disp "INITIAL YEAR  = `initial'"
			use "$dta\appended_countries\\`c'\\`s'_`c'_links_`initial'.dta", clear
			if `initial' != 2016 {
				forval y = `=`initial'+1'(1)2016 {
					preserve 
						use "$dta\appended_countries\\`c'\\`s'_`c'_links_`y'.dta", clear
						qui count
					restore
					if `r(N)' > 1 {
					disp "ADDING YEAR = `y'"
					append using "$dta\appended_countries\\`c'\\`s'_`c'_links_`y'.dta"
					}
					if `r(N)' == 1 {
						disp "NO OBS FOR YEAR = `y'"
					}
				}
				disp "FINAL TABULATION FOR FILE `s' FOR COUNTRY `c'"
					tab file
				save "$dta\appended_countries\\`c'\\`s'_`c'_links_allyrs.dta", replace
			}
			if `initial' == 2016 {
				use "$dta\appended_countries\\`c'\\`s'_`c'_links_2016.dta", clear
					tab file
				save "$dta\appended_countries\\`c'\\`s'_`c'_links_allyrs.dta", replace
			}			
		}
	}
}
